@@ -1,6 +1,6 @@
 # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 # Mobius Forensic Toolkit
-# Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021 Eduardo Aguiar
+# Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021,2022,2023 Eduardo Aguiar
 #
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License as published by the
@@ -15,6 +15,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+import traceback
 import mobius
 import pymobius.operating_system
 
@@ -48,8 +49,8 @@ class Ant (object):
 
     try:
       self.__retrieve_credentials ()
-    except Exception, e:
-      mobius.core.logf ('WRN ant.turing: %s' % str (e))
+    except Exception as e:
+      mobius.core.logf ('WRN %s %s' % (str (e) + ' ' + traceback.format_exc ()))
 
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   # @brief Event <i>on_export_data</i>: Export data to mediator ant
@@ -63,7 +64,7 @@ class Ant (object):
   # @param ant Mediator ant object
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   def on_stop (self, ant):
-    mobius.core.logf ('INF ant.turing: %d/%d Credentials password(s) decrypted' % (self.__count, len (self.__secrets)))
+    mobius.core.logf ('INF %d/%d Credentials password(s) decrypted' % (self.__count, len (self.__secrets)))
 
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   # @brief Event <i>on_key</i>: Process key object
@@ -82,8 +83,8 @@ class Ant (object):
       try:
         if not blob.is_decrypted () and blob.master_key_guid == key.id:
           self.__decrypt_secret (ant, secret, key)
-      except Exception, e:
-        mobius.core.logf ('WRN ant.turing.credentials: %s' % str (e))
+      except Exception as e:
+        mobius.core.logf ('WRN %s %s' % (str (e) + ' ' + traceback.format_exc ()))
 
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   # @brief Try to decrypt secret
@@ -100,7 +101,7 @@ class Ant (object):
         ant.add_password (p)
 
     else:
-      mobius.core.logf ('WRN ant.turing: Credential DPAPI blob not decrypted')
+      mobius.core.logf ('WRN Credential DPAPI blob not decrypted')
 
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   # @brief Retrieve Credentials
@@ -127,77 +128,91 @@ class Ant (object):
             for child in folder.get_children ():
               self.__retrieve_sys_credentials_from_profile (child)
 
-        except Exception, e:
-          mobius.core.logf ('WRN %s' % str (e))
-          traceback.print_exc ()
+        except Exception as e:
+          mobius.core.logf ('WRN %s %s' % (str (e) + ' ' + traceback.format_exc ()))
 
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   # @brief Retrieve System Credentials from profile folder
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   def __retrieve_sys_credentials_from_profile (self, folder):
-    if not folder or folder.is_reallocated () or not folder.is_folder ():
-      return
+    try:
+      if not folder or folder.is_reallocated () or not folder.is_folder ():
+        return
 
-    f = folder.get_child_by_path ('AppData/Local/Microsoft/Credentials')
-    self.__retrieve_credentials_from_folder (f)
+      f = folder.get_child_by_path ('AppData/Local/Microsoft/Credentials')
+      self.__retrieve_credentials_from_folder (f)
 
-    f = folder.get_child_by_path ('AppData/Roaming/Microsoft/Credentials')
-    self.__retrieve_credentials_from_folder (f)
+      f = folder.get_child_by_path ('AppData/Roaming/Microsoft/Credentials')
+      self.__retrieve_credentials_from_folder (f)
+    except Exception as e:
+      mobius.core.logf ('WRN %s %s' % (str (e) + ' ' + traceback.format_exc ()))
 
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   # @brief Retrieve User Credentials
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   def __retrieve_user_credentials (self):
-    for opsys in pymobius.operating_system.scan (self.__item):
-      for user_profile in opsys.get_profiles ():
-        try:
-          folder = user_profile.get_entry_by_path ('%appdata%/Microsoft/Credentials')
-          self.__retrieve_credentials_from_folder (folder)
+    try:
+      for opsys in pymobius.operating_system.scan (self.__item):
+        for user_profile in opsys.get_profiles ():
+          self.__retrieve_credentials_from_profile (user_profile)
+    except Exception as e:
+      mobius.core.logf ('WRN %s %s' % (str (e), traceback.format_exc ()))
 
-          folder = user_profile.get_entry_by_path ('%localappdata%/Microsoft/Credentials')
-          self.__retrieve_credentials_from_folder (folder)
-        except Exception, e:
-          mobius.core.logf ('WRN ant.turing: %s' % str (e))
+  # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+  # @brief Retrieve Credentials from profile folder
+  # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+  def __retrieve_credentials_from_profile (self, user_profile):
+    try:
+      folder = user_profile.get_entry_by_path ('%appdata%/Microsoft/Credentials')
+      self.__retrieve_credentials_from_folder (folder)
+
+      folder = user_profile.get_entry_by_path ('%localappdata%/Microsoft/Credentials')
+      self.__retrieve_credentials_from_folder (folder)
+    except Exception as e:
+      mobius.core.logf ('WRN %s %s' % (str (e), traceback.format_exc ()))
 
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   # @brief Retrieve Credentials from folder
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   def __retrieve_credentials_from_folder (self, folder):
-    if not folder or folder.is_reallocated () or not folder.is_folder ():
-      return
+    try:
+      if not folder or folder.is_reallocated () or not folder.is_folder ():
+        return
 
-    for child in folder.get_children ():
-      try:
+      for child in folder.get_children ():
         self.__retrieve_credentials_from_file (child)
-      except Exception, e:
-        mobius.core.logf ('WRN ant.turing: %s' % str (e))
+    except Exception as e:
+      mobius.core.logf ('WRN %s %s' % (str (e), traceback.format_exc ()))
 
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   # @brief Retrieve Credentials from file
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   def __retrieve_credentials_from_file (self, f):
-    if not f or f.is_reallocated () or not f.is_file ():
-      return
+    try:
+      if not f or f.is_reallocated () or not f.is_file ():
+        return
 
-    reader = f.new_reader ()
+      reader = f.new_reader ()
 
-    if not reader:
-      return
+      if not reader:
+        return
 
-    decoder = mobius.decoder.data_decoder (reader)
-    revision = decoder.get_uint32_le ()
-    blob_size = decoder.get_uint64_le ()
+      decoder = mobius.decoder.data_decoder (reader)
+      revision = decoder.get_uint32_le ()
+      blob_size = decoder.get_uint64_le ()
 
-    if blob_size > 0:
-      blob_data = decoder.get_bytearray_by_size (blob_size)
-      
-      secret = dataholder ()
-      secret.blob = mobius.os.win.dpapi.blob (blob_data)
-      secret.path = f.path.replace ('/', '\\')
+      if blob_size > 0:
+        blob_data = decoder.get_bytearray_by_size (blob_size)
 
-      self.__secrets.append (secret)
+        secret = dataholder ()
+        secret.blob = mobius.os.win.dpapi.blob (blob_data)
+        secret.path = f.path.replace ('/', '\\')
 
-    f.set_handled ()
+        self.__secrets.append (secret)
+
+      f.set_handled ()
+    except Exception as e:
+      mobius.core.logf ('WRN %s %s' % (str (e), traceback.format_exc ()))
 
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   # @brief Create password from Credentials data
@@ -206,22 +221,22 @@ class Ant (object):
     credential = mobius.os.win.credential (data)
 
     # handle blob data
-    blob_data = ''
+    blob_data = b''
     for name, data in credential.data:
       blob_data += data
 
-    if blob_data.startswith ('\x01\x00\x00\x00'):
+    if blob_data.startswith (b'\x01\x00\x00\x00'):
       b = mobius.os.win.dpapi.blob (blob_data)
       mk = self.__master_keys.get (b.master_key_guid)
 
       if mk and b.decrypt (mk):
         blob_data = b.plain_text
       else:
-        mobius.core.logf ('WRN ant.turing: Credential (virtualapp/didlogical) not decrypted')
+        mobius.core.logf ('WRN Credential (virtualapp/didlogical) not decrypted')
 
     if blob_data:
-      mobius.core.logf ('ant.turing: BLOB_DATA')
-      mobius.core.logf (pymobius.dump (blob_data))
+      mobius.core.logf ('DBG BLOB_DATA')
+      mobius.core.logf ('DBG ' + pymobius.dump (blob_data))
 
     # process credential according to domain
     if credential.domain.startswith ('WindowsLive:'):
@@ -229,17 +244,17 @@ class Ant (object):
 
     elif credential.password:
       p = self.__create_password_from_credential_generic (credential)
-      
+
     else:
       p = None
-      mobius.core.logf ('DEV ant.turing: Credential not handled -> ' + credential.domain)
+      mobius.core.logf ('DEV Credential not handled -> ' + credential.domain)
 
       if credential.password_data:
-        mobius.core.logf ('DEV ant.turing: Credential password -> ' + pymobius.dump (credential.password_data))
-        
+        mobius.core.logf ('DEV Credential password -> ' + pymobius.dump (credential.password_data))
+
       for name, value in credential.data:
-        mobius.core.logf ('DEV ant.turing: Credential data -> %s' % name)
-        mobius.core.logf ('DEV ant.turing:' + pymobius.dump (value))
+        mobius.core.logf ('DEV Credential data -> %s' % name)
+        mobius.core.logf ('DEV ' + pymobius.dump (value))
 
     return p
 
@@ -253,7 +268,7 @@ class Ant (object):
     if credential.password:
       p = dataholder ()
       p.type = 'net.account'
-      p.value = credential.password.encode ('utf-8')
+      p.value = credential.password
       p.description = 'Windows Live password. Account: %s' % credential.username
 
       p.metadata = []
@@ -270,12 +285,12 @@ class Ant (object):
   def __create_password_from_credential_generic (self, credential):
     p = dataholder ()
     p.type = 'os.credential'
-    p.value = credential.password.encode ('utf-8')
+    p.value = credential.password
     p.description = 'Win Credential. Domain: ' + credential.domain
 
     p.metadata = []
     p.metadata.append (('Domain', credential.domain))
     p.metadata.append (('Account', credential.username))
     p.metadata.append (('Last update', credential.last_update_time))
-    
+
     return p
