@@ -1,6 +1,6 @@
 # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 # Mobius Forensic Toolkit
-# Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021,2022 Eduardo Aguiar
+# Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021,2022,2023 Eduardo Aguiar
 #
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License as published by the
@@ -18,15 +18,16 @@
 import pymobius.registry
 import pymobius.registry.main
 import pymobius.vfs
-import model
-import ares
-import emuletorrent
-import emule
-import shareaza
 import os.path
-import datetime
 import mobius
 import sys
+import traceback
+from . import model
+from . import ares
+from . import emuletorrent
+from . import emule
+from . import shareaza
+from . import utorrent
 
 # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 # Constants
@@ -36,6 +37,10 @@ STATE_YES = 1
 STATE_ALWAYS = 2
 STATE_UNKNOWN = -1
 
+ANT_ID = 'p2p'
+ANT_NAME = 'P2P Agent'
+ANT_VERSION = '1.0'
+
 # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 # @brief Add file to item's kff ignored file
 # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -105,12 +110,17 @@ class Ant (object):
   # @brief initialize object
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   def __init__ (self, item):
-    if item.datasource:
+    self.__item = item
+    self.__model = None
+
+    if item.has_ant (ANT_ID):
       self.__model = model.model (item)
-      self.__item = item
       self.__is_data_available = self.__model.load ()
+
+    elif item.has_vfs ():
+      self.__is_data_available = False
+
     else:
-      self.__model = None
       self.__is_data_available = True
 
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -136,11 +146,11 @@ class Ant (object):
     if not vfs:
       raise Exception ('Datasource is not available')
 
-    mobius.core.logf ('DBG pymobius.p2p.__retrieve_data (1)')
+    mobius.core.logf ('DBG 1')
     self.__model = model.model (self.__item)
 
     # Clear p2p ignored kff, if any
-    mobius.core.logf ('DBG pymobius.p2p.__retrieve_data (2)')
+    mobius.core.logf ('DBG 2')
     case = self.__item.case
     path = case.get_path (os.path.join ('hashset', '%04d-p2p.ignore' % self.__item.uid))
 
@@ -148,27 +158,29 @@ class Ant (object):
       os.remove (path)
 
     # Retrieve activity data
-    mobius.core.logf ('DBG pymobius.p2p.__retrieve_data (3.1)')
+    mobius.core.logf ('DBG 3.1')
     pymobius.p2p.ares.retrieve (self.__model)
-    mobius.core.logf ('DBG pymobius.p2p.__retrieve_data (3.2)')
+    mobius.core.logf ('DBG 3.2')
     pymobius.p2p.emule.retrieve (self.__model)
-    mobius.core.logf ('DBG pymobius.p2p.__retrieve_data (3.3)')
+    mobius.core.logf ('DBG 3.3')
     pymobius.p2p.emuletorrent.retrieve (self.__model)
-    mobius.core.logf ('DBG pymobius.p2p.__retrieve_data (3.4)')
+    mobius.core.logf ('DBG 3.4')
     pymobius.p2p.shareaza.retrieve (self.__model)
-    mobius.core.logf ('DBG pymobius.p2p.__retrieve_data (3.5)')
+    mobius.core.logf ('DBG 3.5')
+    pymobius.p2p.utorrent.retrieve (self.__model)
 
     # Normalize data
-    mobius.core.logf ('DBG pymobius.p2p.__retrieve_data (4)')
+    mobius.core.logf ('DBG 4')
     self.__calculate_totals ()
     self.__model.accounts = unique ((a, a.network, a.guid) for a in self.__model.accounts)
 
     # Save data model
-    mobius.core.logf ('DBG pymobius.p2p.__retrieve_data (5)')
+    mobius.core.logf ('DBG 5')
     path = self.__item.create_data_path ('p2p.json')
     self.__model.save (path)
     self.__is_data_available = True
-    mobius.core.logf ('DBG pymobius.p2p.__retrieve_data (6)')
+    self.__item.set_ant (ANT_ID, ANT_NAME, ANT_VERSION)
+    mobius.core.logf ('DBG 6')
 
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   # @brief Calculate totals by application
@@ -177,11 +189,11 @@ class Ant (object):
     application = dict ((a.id, a) for a in self.__model.applications)
 
     # Reset totals
-    for app in application.itervalues ():
+    for app in application.values ():
       app.total_searches = 0
       app.total_local_files = 0
       app.total_remote_files = 0
-  
+
     # Calculate totals
     for search in self.__model.searches:
       application[search.app_id].total_searches += 1
