@@ -1,6 +1,6 @@
 # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 # Mobius Forensic Toolkit
-# Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021,2022 Eduardo Aguiar
+# Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021,2022,2023 Eduardo Aguiar
 #
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License as published by the
@@ -15,16 +15,17 @@
 # You should have received a copy of the GNU General Public License
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+import datetime
+import json
+import os
+import os.path
+import traceback
+import mobius
 import pymobius.p2p.account
 import pymobius.p2p.application
 import pymobius.p2p.search
 import pymobius.p2p.local_file
 import pymobius.p2p.remote_file
-import mobius
-import datetime
-import json
-import os
-import os.path
 
 # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 # @brief P2P activity model class
@@ -41,7 +42,6 @@ class model (object):
     self.searches = []
     self.local_files = []
     self.remote_files = []
-    self.mtime = datetime.datetime.utcnow ()
     
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   # @brief load model from .json file
@@ -50,7 +50,7 @@ class model (object):
     rc = False
     path = self.item.get_data_path ('p2p.json')
 
-    if self.item.datasource and os.path.exists (path):
+    if os.path.exists (path):
       fp = open (path)
       data = json.load (fp, object_hook=json_decoder)
       fp.close ()
@@ -60,10 +60,8 @@ class model (object):
       self.searches = data.get ('searches')
       self.local_files = data.get ('local_files')
       self.remote_files = data.get ('remote_files')
-      self.mtime = data.get ('mtime')
 
-      if self.mtime >= self.item.datasource.mtime:
-        rc = True
+      rc = True
 
     return rc
 
@@ -78,17 +76,16 @@ class model (object):
             'searches' : self.searches,
             'local_files' : self.local_files,
             'remote_files' : self.remote_files,
-            'mtime' : self.mtime
             }
 
-    fp = open (path, 'w')
+    f = mobius.io.new_file_by_path (path)
+    fp = mobius.io.text_writer (f.new_writer ())
+
     try:
       json.dump (data, fp, default=json_encoder)
-      fp.close ()
 
-    except Exception, e:
-      mobius.core.logf ('WRN ' + str (e))
-      fp.close ()
+    except Exception as e:
+      mobius.core.logf ('WRN %s %s' % (str (e), traceback.format_exc ()))
       os.remove (path)
     
 # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -102,6 +99,12 @@ def json_encoder (obj):
             'data' : [obj.year, obj.month, obj.day, obj.hour, obj.minute, obj.second]
            }
 
+  elif isinstance (obj, bytes):
+    data = {
+             'class' : 'bytes',
+             'data': mobius.encoder.base64 (obj)
+           }
+
   elif isinstance (obj, pymobius.p2p.account.account):
     data = {
             'class' : 'p2p.account',
@@ -130,6 +133,7 @@ def json_encoder (obj):
             'app' : obj.app,
             'timestamp' : obj.timestamp,
             'username' : obj.username,
+            'count' : obj.count,
             'text' : obj.text,
             'metadata' : list (obj.iter_metadata ()),
             'files' : list (obj.iter_files ())
@@ -149,7 +153,7 @@ def json_encoder (obj):
             'flag_uploaded' : obj.flag_uploaded,
             'flag_completed' : obj.flag_completed,
             'flag_corrupted' : obj.flag_corrupted,
-            'metadata' : obj.get_metadata (),
+            'metadata' : list (obj.get_metadata ()),
             'hashes' : obj.get_hashes ()
            }
 
@@ -165,7 +169,7 @@ def json_encoder (obj):
             'url' : obj.url,
             'preview_data' : obj.preview_data,
             'peer' : obj.peer,
-            'metadata' : obj.get_metadata (),
+            'metadata' : list (obj.get_metadata ()),
             'hashes' : obj.get_hashes ()
            }
 
@@ -181,7 +185,7 @@ def json_encoder (obj):
            }
 
   else:
-    raise TypeError (repr (obj) + " is not JSON serializable")
+    raise TypeError ("%s type is not JSON serializable" % type (obj))
 
   return data
 
@@ -195,6 +199,9 @@ def json_decoder (dct):
     args = dct.get ('data')
     data = datetime.datetime (*args)
 
+  elif classname == 'bytes':
+    data = mobius.decoder.base64 (dct.get ('data'))
+
   elif classname == 'p2p.account':
     data = pymobius.p2p.account.account ()
     data.app_id = dct.get ('app_id')
@@ -266,6 +273,7 @@ def json_decoder (dct):
     data.timestamp = dct.get ('timestamp')
     data.username = dct.get ('username')
     data.text = dct.get ('text')
+    data.count = dct.get ('count')
     
     for name, value in dct.get ('metadata', []):
       data.add_metadata (name, value)
